@@ -135,24 +135,38 @@ enum {
 };
 
 /**
- * Flags (4th arg) to HX_getopt.
- * %HXOPT_PTHRU:	pass-through unknown options to new argv
+ * Flags to HX_getopt.
+ * %HXOPT_PTHRU:	pass-through unknown options to new argv (obsolete)
  * %HXOPT_DESTROY_OLD:	destroy old argv after parsing is successful
  * %HXOPT_QUIET:	do not output any warnings to stderr
  * %HXOPT_HELPONERR:	print out help when a parsing error occurs
  * %HXOPT_USAGEONERR:	print out short usage when a parsing error occurs
- * %HXOPT_RQ_ORDER:	require option order/POSIX mode:
- * 			first non-option terminates option processing
+ * %HXOPT_RQ_ORDER:     Options and non-options must not be mixed (first
+ *                      non-option stops parsing) and the environment variable
+ *                      POSIXLY_CORRECT is ignored.
  * %HXOPT_KEEP_ARGV:	do not replace argc/argv at all
+ * %HXOPT_ANY_ORDER:    Options and non-options may be mixed and the
+ *                      environment variable POSIXLY_CORRECT is ignored.
+ * %HXOPT_CONST_INPUT:   User declaration that elements in input argv can NOT
+ *                       be reordered by the parser.
+ * %HXOPT_ITER_OPTS:    (HX_getopt6 only) Populate result.desc and .oarg.
+ * %HXOPT_ITER_ARGS:    (HX_getopt6 only) Populate result.uarg.
+ * %HXOPT_DUP_ARGS:     (HX_getopt6 only) Populate result.dup_argv.
  */
 enum {
-	HXOPT_PTHRU       = 1 << 0,
-	HXOPT_DESTROY_OLD = 1 << 1,
-	HXOPT_QUIET       = 1 << 2,
-	HXOPT_HELPONERR   = 1 << 3,
-	HXOPT_USAGEONERR  = 1 << 4,
-	HXOPT_RQ_ORDER    = 1 << 5,
-	HXOPT_KEEP_ARGV   = 1 << 6,
+	HXOPT_PTHRU       = 0x1U,
+	HXOPT_DESTROY_OLD = 0x2U,
+	HXOPT_QUIET       = 0x4U,
+	HXOPT_HELPONERR   = 0x8U,
+	HXOPT_USAGEONERR  = 0x10U,
+	HXOPT_RQ_ORDER    = 0x20U,
+	HXOPT_KEEP_ARGV   = 0x40U,
+	HXOPT_ANY_ORDER   = 0x80U,
+	HXOPT_CONST_INPUT   = 0x100U,
+	HXOPT_ITER_OPTS   = 0x200U,
+	HXOPT_ITER_ARGS   = 0x400U,
+	HXOPT_ITER_OA     = HXOPT_ITER_OPTS | HXOPT_ITER_ARGS,
+	HXOPT_DUP_ARGS    = 0x800U,
 };
 
 /**
@@ -162,6 +176,7 @@ enum {
  * %HXOPT_ERR_VOID:	long option takes no value
  * %HXOPT_ERR_MIS:	option requires a value argument
  * %HXOPT_ERR_AMBIG:	long option abbreviation was ambiguous
+ * %HXOPT_ERR_FLAGS:    illegal flag combination (API misuse)
  */
 enum {
 	HXOPT_ERR_SUCCESS = 0,
@@ -169,6 +184,7 @@ enum {
 	HXOPT_ERR_VOID,
 	HXOPT_ERR_MIS,
 	HXOPT_ERR_AMBIG,
+	HXOPT_ERR_FLAGS,
 };
 
 /**
@@ -229,12 +245,34 @@ struct HXoption {
 	const char *help, *htyp;
 };
 
+/**
+ * @nopts:    Number of options found.
+ * @nargs:    Number of non-option arguments found.
+ * @desc:     Under %HXOPT_ITER_OPTS, filled with pointers to options, else %nullptr.
+ * @oarg:     Under %HXOPT_ITER_OPTS, filled with pointers to optargs, else %nullptr.
+ * @uarg:     Under %HXOPT_ITER_ARGS, filled with pointers to non-opt args
+ *            (always orig_argv, never dup_argv), else %nullptr.
+ * @dup_argc: String count for dup_argv.
+ * @dup_argv: Filled with copies of non-option arguments if %HXOPT_DUP_ARGS.
+ *            dup_argv[0] will be the program name (useful for feeding to
+ *            another HX_getopt6 call).
+ */
+struct HXopt6_result {
+	int nopts, nargs;
+	const struct HXoption **desc;
+	char **oarg, **uarg;
+	int dup_argc;
+	char **dup_argv;
+};
+
 #ifndef LIBHX_ZVECFREE_DECLARATION
 #define LIBHX_ZVECFREE_DECLARATION
 extern void HX_zvecfree(char **);
 #endif
 extern int HX_getopt(const struct HXoption *, int *, char ***, unsigned int);
 extern int HX_getopt5(const struct HXoption *, char **argv, int *nargc, char ***nargv, unsigned int flags);
+extern int HX_getopt6(const struct HXoption *, int, char **argv, struct HXopt6_result *, unsigned int flags);
+extern void HX_getopt6_clean(struct HXopt6_result *);
 #define HX_getopt(a, b, c, d) HX_getopt((a), (b), const_cast3(char ***, (c)), (d))
 extern void HX_getopt_help(const struct HXoptcb *, FILE *);
 extern void HX_getopt_help_cb(const struct HXoptcb *);
@@ -265,6 +303,20 @@ extern void HX_shconfig_free(const struct HXoption *);
 		{"usage", 0, HXTYPE_NONE, NULL, NULL, HX_getopt_usage_cb, \
 		0, "Display brief usage message"}
 #	define HXOPT_TABLEEND {NULL, 0, HXTYPE_XSNTMARK}
+
+struct HXopt6_auto_result : public HXopt6_result {
+	HXopt6_auto_result() : HXopt6_result() {}
+	~HXopt6_auto_result() { HX_getopt6_clean(this); }
+	/*
+	 * Unlike the unique_tie class, no operator~ is provided for
+	 * HXopt6_auto_result, because reusing a result is not outright
+	 * possible;
+	 * auto ret = HX_getopt6(&table, argc, argv, &result, HXOPT_ITER_ARGS);
+	 * ret = HX_getopt6(&table, result.nargs, result.uarg, &~result, HXOPT_ITER_ARGS):
+	 * would kill off result before nargs/uarg is loaded.
+	 */
+};
+
 #endif
 
 #endif /* _LIBHX_OPTION_H */
